package java.time.format;

import java.time.Instant;
import java.time.ZoneOffset;
import java.util.Iterator;
import java.time.chrono.ChronoLocalDateTime;
import java.time.DateTimeException;
import java.time.chrono.ChronoZonedDateTime;
import java.util.Set;
import java.time.LocalDate;
import java.time.temporal.TemporalQueries;
import java.time.temporal.TemporalQuery;
import java.time.temporal.UnsupportedTemporalTypeException;
import java.util.Objects;
import java.time.temporal.ChronoField;
import java.util.HashMap;
import java.time.Period;
import java.time.LocalTime;
import java.time.chrono.ChronoLocalDate;
import java.time.chrono.Chronology;
import java.time.ZoneId;
import java.time.temporal.TemporalField;
import java.util.Map;
import java.time.temporal.TemporalAccessor;

final class Parsed implements TemporalAccessor
{
    final Map<TemporalField, Long> fieldValues;
    ZoneId zone;
    Chronology chrono;
    boolean leapSecond;
    private ResolverStyle resolverStyle;
    private ChronoLocalDate date;
    private LocalTime time;
    Period excessDays;
    
    Parsed() {
        this.fieldValues = new HashMap();
        this.excessDays = Period.ZERO;
    }
    
    Parsed copy() {
        Parsed parsed = new Parsed();
        parsed.fieldValues.putAll(this.fieldValues);
        parsed.zone = this.zone;
        parsed.chrono = this.chrono;
        parsed.leapSecond = this.leapSecond;
        return parsed;
    }
    
    @Override
    public boolean isSupported(TemporalField temporalField) {
        return this.fieldValues.containsKey(temporalField) || (this.date != null && this.date.isSupported(temporalField)) || (this.time != null && this.time.isSupported(temporalField)) || (temporalField != null && !(temporalField instanceof ChronoField) && temporalField.isSupportedBy(this));
    }
    
    @Override
    public long getLong(TemporalField field) {
        Objects.requireNonNull(field, "field");
        Long n = (Long)this.fieldValues.get(field);
        if (n != null) {
            return (long)n;
        }
        if (this.date != null && this.date.isSupported(field)) {
            return this.date.getLong(field);
        }
        if (this.time != null && this.time.isSupported(field)) {
            return this.time.getLong(field);
        }
        if ((field instanceof ChronoField)) {
            throw new UnsupportedTemporalTypeException("Unsupported field: " + field);
        }
        return field.getFrom(this);
    }
    
    @Override
    public <R> R query(TemporalQuery<R> temporalQuery) {
        if (temporalQuery == TemporalQueries.zoneId()) {
            return this.zone;
        }
        if (temporalQuery == TemporalQueries.chronology()) {
            return this.chrono;
        }
        if (temporalQuery == TemporalQueries.localDate()) {
            return (this.date != null) ? LocalDate.from(this.date) : null;
        }
        if (temporalQuery == TemporalQueries.localTime()) {
            return this.time;
        }
        if (temporalQuery == TemporalQueries.zone() || temporalQuery == TemporalQueries.offset()) {
            return temporalQuery.queryFrom(this);
        }
        if (temporalQuery == TemporalQueries.precision()) {
            return null;
        }
        return temporalQuery.queryFrom(this);
    }
    
    TemporalAccessor resolve(ResolverStyle resolverStyle, Set<TemporalField> set) {
        if (set != null) {
            this.fieldValues.keySet().retainAll(set);
        }
        this.resolverStyle = resolverStyle;
        this.resolveFields();
        this.resolveTimeLenient();
        this.crossCheck();
        this.resolvePeriod();
        this.resolveFractional();
        this.resolveInstant();
        return this;
    }
    
    private void resolveFields() {
        this.resolveInstantFields();
        this.resolveDateFields();
        this.resolveTimeFields();
        if (this.fieldValues.size() > 0) {
            int i;
            for (i = 0; i < 50; ++i) {
                Iterator<Map.Entry<TemporalField, Long>> iterator = this.fieldValues.entrySet().iterator();
                while (iterator.hasNext()) {
                    TemporalField temporalField = (TemporalField)((Map.Entry)iterator.next()).getKey();
                    TemporalAccessor temporalAccessor = temporalField.resolve(this.fieldValues, this, this.resolverStyle);
                    if (temporalAccessor != null) {
                        if ((temporalAccessor instanceof ChronoZonedDateTime)) {
                            ChronoZonedDateTime<Object> chronoZonedDateTime = (ChronoZonedDateTime)temporalAccessor;
                            if (this.zone == null) {
                                this.zone = chronoZonedDateTime.getZone();
                            }
                            else if (!this.zone.equals(chronoZonedDateTime.getZone())) {
                                throw new DateTimeException("ChronoZonedDateTime must use the effective parsed zone: " + this.zone);
                            }
                            temporalAccessor = chronoZonedDateTime.toLocalDateTime();
                        }
                        if ((temporalAccessor instanceof ChronoLocalDateTime)) {
                            ChronoLocalDateTime<Object> chronoLocalDateTime = (ChronoLocalDateTime)temporalAccessor;
                            this.updateCheckConflict(chronoLocalDateTime.toLocalTime(), Period.ZERO);
                            this.updateCheckConflict(chronoLocalDateTime.toLocalDate());
                            break;
                        }
                        if ((temporalAccessor instanceof ChronoLocalDate)) {
                            this.updateCheckConflict((ChronoLocalDate)temporalAccessor);
                            break;
                        }
                        if ((temporalAccessor instanceof LocalTime)) {
                            this.updateCheckConflict((LocalTime)temporalAccessor, Period.ZERO);
                            break;
                        }
                        throw new DateTimeException("Method resolve() can only return ChronoZonedDateTime, ChronoLocalDateTime, ChronoLocalDate or LocalTime");
                    }
                    else {
                        if (!this.fieldValues.containsKey(temporalField)) {
                            break;
                        }
                        continue;
                    }
                }
            }
            if (i == 50) {
                throw new DateTimeException("One of the parsed fields has an incorrectly implemented resolve method");
            }
            if (i > 0) {
                this.resolveInstantFields();
                this.resolveDateFields();
                this.resolveTimeFields();
            }
        }
    }
    
    private void updateCheckConflict(TemporalField obj, TemporalField temporalField, Long obj2) {
        Long obj3 = (Long)this.fieldValues.put(temporalField, obj2);
        if (obj3 != null && (long)obj3 != (long)obj2) {
            throw new DateTimeException("Conflict found: " + temporalField + " " + obj3 + " differs from " + temporalField + " " + obj2 + " while resolving  " + obj);
        }
        return;
    }
    
    private void resolveInstantFields() {
        if (this.fieldValues.containsKey(ChronoField.INSTANT_SECONDS)) {
            if (this.zone != null) {
                this.resolveInstantFields0(this.zone);
            }
            else {
                Long n = (Long)this.fieldValues.get(ChronoField.OFFSET_SECONDS);
                if (n != null) {
                    this.resolveInstantFields0(ZoneOffset.ofTotalSeconds(n.intValue()));
                }
            }
        }
    }
    
    private void resolveInstantFields0(ZoneId zone) {
        ChronoZonedDateTime<Object> zonedDateTime = this.chrono.zonedDateTime(Instant.ofEpochSecond((long)this.fieldValues.remove(ChronoField.INSTANT_SECONDS)), zone);
        this.updateCheckConflict(zonedDateTime.toLocalDate());
        this.updateCheckConflict(ChronoField.INSTANT_SECONDS, ChronoField.SECOND_OF_DAY, Long.valueOf((long)zonedDateTime.toLocalTime().toSecondOfDay()));
    }
    
    private void resolveDateFields() {
        this.updateCheckConflict(this.chrono.resolveDate(this.fieldValues, this.resolverStyle));
    }
    
    private void updateCheckConflict(ChronoLocalDate chronoLocalDate) {
        if (this.date != null) {
            if (chronoLocalDate != null && !this.date.equals(chronoLocalDate)) {
                throw new DateTimeException("Conflict found: Fields resolved to two different dates: " + this.date + " " + chronoLocalDate);
            }
        }
        else if (chronoLocalDate != null) {
            if (!this.chrono.equals(chronoLocalDate.getChronology())) {
                throw new DateTimeException("ChronoLocalDate must use the effective parsed chronology: " + this.chrono);
            }
            this.date = chronoLocalDate;
        }
    }
    
    private void resolveTimeFields() {
        if (this.fieldValues.containsKey(ChronoField.CLOCK_HOUR_OF_DAY)) {
            long longValue = (long)this.fieldValues.remove(ChronoField.CLOCK_HOUR_OF_DAY);
            if (this.resolverStyle == ResolverStyle.STRICT || (this.resolverStyle == ResolverStyle.SMART && longValue != 0L)) {
                ChronoField.CLOCK_HOUR_OF_DAY.checkValidValue(longValue);
            }
            this.updateCheckConflict(ChronoField.CLOCK_HOUR_OF_DAY, ChronoField.HOUR_OF_DAY, Long.valueOf((longValue == 24L) ? 0L : longValue));
        }
        if (this.fieldValues.containsKey(ChronoField.CLOCK_HOUR_OF_AMPM)) {
            long longValue2 = (long)this.fieldValues.remove(ChronoField.CLOCK_HOUR_OF_AMPM);
            if (this.resolverStyle == ResolverStyle.STRICT || (this.resolverStyle == ResolverStyle.SMART && longValue2 != 0L)) {
                ChronoField.CLOCK_HOUR_OF_AMPM.checkValidValue(longValue2);
            }
            this.updateCheckConflict(ChronoField.CLOCK_HOUR_OF_AMPM, ChronoField.HOUR_OF_AMPM, Long.valueOf((longValue2 == 12L) ? 0L : longValue2));
        }
        if (this.fieldValues.containsKey(ChronoField.AMPM_OF_DAY) && this.fieldValues.containsKey(ChronoField.HOUR_OF_AMPM)) {
            long longValue3 = (long)this.fieldValues.remove(ChronoField.AMPM_OF_DAY);
            long longValue4 = (long)this.fieldValues.remove(ChronoField.HOUR_OF_AMPM);
            if (this.resolverStyle == ResolverStyle.LENIENT) {
                this.updateCheckConflict(ChronoField.AMPM_OF_DAY, ChronoField.HOUR_OF_DAY, Long.valueOf(Math.addExact(Math.multiplyExact(longValue3, 12L), longValue4)));
            }
            else {
                ChronoField.AMPM_OF_DAY.checkValidValue(longValue3);
                ChronoField.HOUR_OF_AMPM.checkValidValue(longValue3);
                this.updateCheckConflict(ChronoField.AMPM_OF_DAY, ChronoField.HOUR_OF_DAY, Long.valueOf(longValue3 * 12L + longValue4));
            }
        }
        if (this.fieldValues.containsKey(ChronoField.NANO_OF_DAY)) {
            long longValue5 = (long)this.fieldValues.remove(ChronoField.NANO_OF_DAY);
            if (this.resolverStyle != ResolverStyle.LENIENT) {
                ChronoField.NANO_OF_DAY.checkValidValue(longValue5);
            }
            this.updateCheckConflict(ChronoField.NANO_OF_DAY, ChronoField.HOUR_OF_DAY, Long.valueOf(longValue5 / 3600000000000L));
            this.updateCheckConflict(ChronoField.NANO_OF_DAY, ChronoField.MINUTE_OF_HOUR, Long.valueOf(longValue5 / 60000000000L % 60L));
            this.updateCheckConflict(ChronoField.NANO_OF_DAY, ChronoField.SECOND_OF_MINUTE, Long.valueOf(longValue5 / 1000000000L % 60L));
            this.updateCheckConflict(ChronoField.NANO_OF_DAY, ChronoField.NANO_OF_SECOND, Long.valueOf(longValue5 % 1000000000L));
        }
        if (this.fieldValues.containsKey(ChronoField.MICRO_OF_DAY)) {
            long longValue6 = (long)this.fieldValues.remove(ChronoField.MICRO_OF_DAY);
            if (this.resolverStyle != ResolverStyle.LENIENT) {
                ChronoField.MICRO_OF_DAY.checkValidValue(longValue6);
            }
            this.updateCheckConflict(ChronoField.MICRO_OF_DAY, ChronoField.SECOND_OF_DAY, Long.valueOf(longValue6 / 1000000L));
            this.updateCheckConflict(ChronoField.MICRO_OF_DAY, ChronoField.MICRO_OF_SECOND, Long.valueOf(longValue6 % 1000000L));
        }
        if (this.fieldValues.containsKey(ChronoField.MILLI_OF_DAY)) {
            long longValue7 = (long)this.fieldValues.remove(ChronoField.MILLI_OF_DAY);
            if (this.resolverStyle != ResolverStyle.LENIENT) {
                ChronoField.MILLI_OF_DAY.checkValidValue(longValue7);
            }
            this.updateCheckConflict(ChronoField.MILLI_OF_DAY, ChronoField.SECOND_OF_DAY, Long.valueOf(longValue7 / 1000L));
            this.updateCheckConflict(ChronoField.MILLI_OF_DAY, ChronoField.MILLI_OF_SECOND, Long.valueOf(longValue7 % 1000L));
        }
        if (this.fieldValues.containsKey(ChronoField.SECOND_OF_DAY)) {
            long longValue8 = (long)this.fieldValues.remove(ChronoField.SECOND_OF_DAY);
            if (this.resolverStyle != ResolverStyle.LENIENT) {
                ChronoField.SECOND_OF_DAY.checkValidValue(longValue8);
            }
            this.updateCheckConflict(ChronoField.SECOND_OF_DAY, ChronoField.HOUR_OF_DAY, Long.valueOf(longValue8 / 3600L));
            this.updateCheckConflict(ChronoField.SECOND_OF_DAY, ChronoField.MINUTE_OF_HOUR, Long.valueOf(longValue8 / 60L % 60L));
            this.updateCheckConflict(ChronoField.SECOND_OF_DAY, ChronoField.SECOND_OF_MINUTE, Long.valueOf(longValue8 % 60L));
        }
        if (this.fieldValues.containsKey(ChronoField.MINUTE_OF_DAY)) {
            long longValue9 = (long)this.fieldValues.remove(ChronoField.MINUTE_OF_DAY);
            if (this.resolverStyle != ResolverStyle.LENIENT) {
                ChronoField.MINUTE_OF_DAY.checkValidValue(longValue9);
            }
            this.updateCheckConflict(ChronoField.MINUTE_OF_DAY, ChronoField.HOUR_OF_DAY, Long.valueOf(longValue9 / 60L));
            this.updateCheckConflict(ChronoField.MINUTE_OF_DAY, ChronoField.MINUTE_OF_HOUR, Long.valueOf(longValue9 % 60L));
        }
        if (this.fieldValues.containsKey(ChronoField.NANO_OF_SECOND)) {
            long longValue10 = (long)this.fieldValues.get(ChronoField.NANO_OF_SECOND);
            if (this.resolverStyle != ResolverStyle.LENIENT) {
                ChronoField.NANO_OF_SECOND.checkValidValue(longValue10);
            }
            if (this.fieldValues.containsKey(ChronoField.MICRO_OF_SECOND)) {
                long longValue11 = (long)this.fieldValues.remove(ChronoField.MICRO_OF_SECOND);
                if (this.resolverStyle != ResolverStyle.LENIENT) {
                    ChronoField.MICRO_OF_SECOND.checkValidValue(longValue11);
                }
                longValue10 = longValue11 * 1000L + longValue10 % 1000L;
                this.updateCheckConflict(ChronoField.MICRO_OF_SECOND, ChronoField.NANO_OF_SECOND, Long.valueOf(longValue10));
            }
            if (this.fieldValues.containsKey(ChronoField.MILLI_OF_SECOND)) {
                long longValue12 = (long)this.fieldValues.remove(ChronoField.MILLI_OF_SECOND);
                if (this.resolverStyle != ResolverStyle.LENIENT) {
                    ChronoField.MILLI_OF_SECOND.checkValidValue(longValue12);
                }
                this.updateCheckConflict(ChronoField.MILLI_OF_SECOND, ChronoField.NANO_OF_SECOND, Long.valueOf(longValue12 * 1000000L + longValue10 % 1000000L));
            }
        }
        if (this.fieldValues.containsKey(ChronoField.HOUR_OF_DAY) && this.fieldValues.containsKey(ChronoField.MINUTE_OF_HOUR) && this.fieldValues.containsKey(ChronoField.SECOND_OF_MINUTE) && this.fieldValues.containsKey(ChronoField.NANO_OF_SECOND)) {
            this.resolveTime((long)this.fieldValues.remove(ChronoField.HOUR_OF_DAY), (long)this.fieldValues.remove(ChronoField.MINUTE_OF_HOUR), (long)this.fieldValues.remove(ChronoField.SECOND_OF_MINUTE), (long)this.fieldValues.remove(ChronoField.NANO_OF_SECOND));
        }
    }
    
    private void resolveTimeLenient() {
        if (this.time == null) {
            if (this.fieldValues.containsKey(ChronoField.MILLI_OF_SECOND)) {
                long longValue = (long)this.fieldValues.remove(ChronoField.MILLI_OF_SECOND);
                if (this.fieldValues.containsKey(ChronoField.MICRO_OF_SECOND)) {
                    long l = longValue * 1000L + (long)this.fieldValues.get(ChronoField.MICRO_OF_SECOND) % 1000L;
                    this.updateCheckConflict(ChronoField.MILLI_OF_SECOND, ChronoField.MICRO_OF_SECOND, Long.valueOf(l));
                    this.fieldValues.remove(ChronoField.MICRO_OF_SECOND);
                    this.fieldValues.put(ChronoField.NANO_OF_SECOND, Long.valueOf(l * 1000L));
                }
                else {
                    this.fieldValues.put(ChronoField.NANO_OF_SECOND, Long.valueOf(longValue * 1000000L));
                }
            }
            else if (this.fieldValues.containsKey(ChronoField.MICRO_OF_SECOND)) {
                this.fieldValues.put(ChronoField.NANO_OF_SECOND, Long.valueOf((long)this.fieldValues.remove(ChronoField.MICRO_OF_SECOND) * 1000L));
            }
            Long n = (Long)this.fieldValues.get(ChronoField.HOUR_OF_DAY);
            if (n != null) {
                Long n2 = (Long)this.fieldValues.get(ChronoField.MINUTE_OF_HOUR);
                Long n3 = (Long)this.fieldValues.get(ChronoField.SECOND_OF_MINUTE);
                Long n4 = (Long)this.fieldValues.get(ChronoField.NANO_OF_SECOND);
                if ((n2 == null && (n3 != null || n4 != null)) || (n2 != null && n3 == null && n4 != null)) {
                    return;
                }
                this.resolveTime((long)n, (n2 != null) ? ((long)n2) : 0L, (n3 != null) ? ((long)n3) : 0L, (n4 != null) ? ((long)n4) : 0L);
                this.fieldValues.remove(ChronoField.HOUR_OF_DAY);
                this.fieldValues.remove(ChronoField.MINUTE_OF_HOUR);
                this.fieldValues.remove(ChronoField.SECOND_OF_MINUTE);
                this.fieldValues.remove(ChronoField.NANO_OF_SECOND);
            }
        }
        if (this.resolverStyle != ResolverStyle.LENIENT && this.fieldValues.size() > 0) {
            for (Map.Entry entry : this.fieldValues.entrySet()) {
                TemporalField temporalField = (TemporalField)entry.getKey();
                if (temporalField instanceof ChronoField && temporalField.isTimeBased()) {
                    ((ChronoField)temporalField).checkValidValue((long)entry.getValue());
                }
                else {
                    continue;
                }
            }
        }
    }
    
    private void resolveTime(long n, long n2, long n3, long n4) {
        if (this.resolverStyle == ResolverStyle.LENIENT) {
            long addExact = Math.addExact(Math.addExact(Math.addExact(Math.multiplyExact(n, 3600000000000L), Math.multiplyExact(n2, 60000000000L)), Math.multiplyExact(n3, 1000000000L)), n4);
            this.updateCheckConflict(LocalTime.ofNanoOfDay(Math.floorMod(addExact, 86400000000000L)), Period.ofDays((int)Math.floorDiv(addExact, 86400000000000L)));
        }
        else {
            int checkValidIntValue = ChronoField.MINUTE_OF_HOUR.checkValidIntValue(n2);
            int checkValidIntValue2 = ChronoField.NANO_OF_SECOND.checkValidIntValue(n4);
            if (this.resolverStyle == ResolverStyle.SMART && n == 24L && checkValidIntValue == 0 && n3 == 0L && checkValidIntValue2 == 0) {
                this.updateCheckConflict(LocalTime.MIDNIGHT, Period.ofDays(1));
            }
            else {
                this.updateCheckConflict(LocalTime.of(ChronoField.HOUR_OF_DAY.checkValidIntValue(n), checkValidIntValue, ChronoField.SECOND_OF_MINUTE.checkValidIntValue(n3), checkValidIntValue2), Period.ZERO);
            }
        }
    }
    
    private void resolvePeriod() {
        if (this.date != null && this.time != null && !this.excessDays.isZero()) {
            this.date = this.date.plus(this.excessDays);
            this.excessDays = Period.ZERO;
        }
    }
    
    private void resolveFractional() {
        if (this.time == null && (this.fieldValues.containsKey(ChronoField.INSTANT_SECONDS) || this.fieldValues.containsKey(ChronoField.SECOND_OF_DAY) || this.fieldValues.containsKey(ChronoField.SECOND_OF_MINUTE))) {
            if (this.fieldValues.containsKey(ChronoField.NANO_OF_SECOND)) {
                long longValue = (long)this.fieldValues.get(ChronoField.NANO_OF_SECOND);
                this.fieldValues.put(ChronoField.MICRO_OF_SECOND, Long.valueOf(longValue / 1000L));
                this.fieldValues.put(ChronoField.MILLI_OF_SECOND, Long.valueOf(longValue / 1000000L));
            }
            else {
                this.fieldValues.put(ChronoField.NANO_OF_SECOND, Long.valueOf(0L));
                this.fieldValues.put(ChronoField.MICRO_OF_SECOND, Long.valueOf(0L));
                this.fieldValues.put(ChronoField.MILLI_OF_SECOND, Long.valueOf(0L));
            }
        }
    }
    
    private void resolveInstant() {
        if (this.date != null && this.time != null) {
            if (this.zone != null) {
                this.fieldValues.put(ChronoField.INSTANT_SECONDS, Long.valueOf(this.date.atTime(this.time).atZone(this.zone).getLong(ChronoField.INSTANT_SECONDS)));
            }
            else {
                Long n = (Long)this.fieldValues.get(ChronoField.OFFSET_SECONDS);
                if (n != null) {
                    this.fieldValues.put(ChronoField.INSTANT_SECONDS, Long.valueOf(this.date.atTime(this.time).atZone(ZoneOffset.ofTotalSeconds(n.intValue())).getLong(ChronoField.INSTANT_SECONDS)));
                }
            }
        }
    }
    
    private void updateCheckConflict(LocalTime obj, Period period) {
        if (this.time != null) {
            if (!this.time.equals(obj)) {
                throw new DateTimeException("Conflict found: Fields resolved to different times: " + this.time + " " + obj);
            }
            if (!this.excessDays.isZero() && !period.isZero() && !this.excessDays.equals(period)) {
                throw new DateTimeException("Conflict found: Fields resolved to different excess periods: " + this.excessDays + " " + period);
            }
            this.excessDays = period;
        }
        else {
            this.time = obj;
            this.excessDays = period;
        }
    }
    
    private void crossCheck() {
        if (this.date != null) {
            this.crossCheck(this.date);
        }
        if (this.time != null) {
            this.crossCheck(this.time);
            if (this.date != null && this.fieldValues.size() > 0) {
                this.crossCheck(this.date.atTime(this.time));
            }
        }
    }
    
    private void crossCheck(TemporalAccessor obj) {
        Iterator<Map.Entry<TemporalField, Long>> iterator = this.fieldValues.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<Object, Object> entry = (Map.Entry)iterator.next();
            TemporalField temporalField = (TemporalField)entry.getKey();
            if (obj.isSupported(temporalField)) {
                long long1;
                try {
                    long1 = obj.getLong(temporalField);
                }
                catch (RuntimeException ex) {
                    continue;
                }
                long longValue = (long)entry.getValue();
                if (long1 != longValue) {
                    throw new DateTimeException("Conflict found: Field " + temporalField + " " + long1 + " differs from " + temporalField + " " + longValue + " derived from " + obj);
                }
                iterator.remove();
            }
            else {
                continue;
            }
        }
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(64);
        sb.append(this.fieldValues).append(',').append(this.chrono);
        if (this.zone != null) {
            sb.append(',').append(this.zone);
        }
        if (this.date != null || this.time != null) {
            sb.append(" resolved to ");
            if (this.date != null) {
                sb.append(this.date);
                if (this.time != null) {
                    sb.append('T').append(this.time);
                }
            }
            else {
                sb.append(this.time);
            }
        }
        return sb.toString();
    }
}
